@@ -31,6 +31,8 @@
 >>> L.fold ((,) <$> L.minimum <*> L.maximum) [1..10000000]
 (Just 1,Just 10000000)
 
+    You might want to try enabling the @-flate-dmd-anal@ flag when compiling
+    executables that use this library to further improve performance.
 -}
 
 {-# LANGUAGE ExistentialQuantification #-}
@@ -63,6 +65,9 @@ module Control.Foldl (
     , any
     , sum
     , product
+    , mean
+    , variance
+    , std
     , maximum
     , maximumBy
     , minimum
@@ -71,6 +76,7 @@ module Control.Foldl (
     , notElem
     , find
     , index
+    , lookup
     , elemIndex
     , findIndex
     , random
@@ -105,9 +111,11 @@ module Control.Foldl (
     , premapM
     , Handler
     , handles
+    , foldOver
     , EndoM(..)
     , HandlerM
     , handlesM
+    , foldOverM
     , folded
     , filtered
 
@@ -147,6 +155,7 @@ import Prelude hiding
     , minimum
     , elem
     , notElem
+    , lookup
     )
 
 import qualified Data.Foldable               as F
@@ -181,10 +190,10 @@ instance Profunctor Fold where
 
 instance Comonad (Fold a) where
     extract (Fold _ begin done) = done begin
-    {-#  INLINABLE extract #-}
+    {-#  INLINE extract #-}
 
     duplicate (Fold step begin done) = Fold step begin (\x -> Fold step x done)
-    {-#  INLINABLE duplicate #-}
+    {-#  INLINE duplicate #-}
 
 instance Applicative (Fold a) where
     pure b    = Fold (\() _ -> ()) () (\() -> b)
@@ -199,97 +208,97 @@ instance Applicative (Fold a) where
 
 instance Monoid b => Monoid (Fold a b) where
     mempty = pure mempty
-    {-# INLINABLE mempty #-}
+    {-# INLINE mempty #-}
 
     mappend = liftA2 mappend
-    {-# INLINABLE mappend #-}
+    {-# INLINE mappend #-}
 
 instance Num b => Num (Fold a b) where
     fromInteger = pure . fromInteger
-    {-# INLINABLE fromInteger #-}
+    {-# INLINE fromInteger #-}
 
     negate = fmap negate
-    {-# INLINABLE negate #-}
+    {-# INLINE negate #-}
 
     abs = fmap abs
-    {-# INLINABLE abs #-}
+    {-# INLINE abs #-}
 
     signum = fmap signum
-    {-# INLINABLE signum #-}
+    {-# INLINE signum #-}
 
     (+) = liftA2 (+)
-    {-# INLINABLE (+) #-}
+    {-# INLINE (+) #-}
 
     (*) = liftA2 (*)
-    {-# INLINABLE (*) #-}
+    {-# INLINE (*) #-}
 
     (-) = liftA2 (-)
-    {-# INLINABLE (-) #-}
+    {-# INLINE (-) #-}
 
 instance Fractional b => Fractional (Fold a b) where
     fromRational = pure . fromRational
-    {-# INLINABLE fromRational #-}
+    {-# INLINE fromRational #-}
 
     recip = fmap recip
-    {-# INLINABLE recip #-}
+    {-# INLINE recip #-}
 
     (/) = liftA2 (/)
-    {-# INLINABLE (/) #-}
+    {-# INLINE (/) #-}
 
 instance Floating b => Floating (Fold a b) where
     pi = pure pi
-    {-# INLINABLE pi #-}
+    {-# INLINE pi #-}
 
     exp = fmap exp
-    {-# INLINABLE exp #-}
+    {-# INLINE exp #-}
 
     sqrt = fmap sqrt
-    {-# INLINABLE sqrt #-}
+    {-# INLINE sqrt #-}
 
     log = fmap log
-    {-# INLINABLE log #-}
+    {-# INLINE log #-}
 
     sin = fmap sin
-    {-# INLINABLE sin #-}
+    {-# INLINE sin #-}
 
     tan = fmap tan
-    {-# INLINABLE tan #-}
+    {-# INLINE tan #-}
 
     cos = fmap cos
-    {-# INLINABLE cos #-}
+    {-# INLINE cos #-}
 
     asin = fmap sin
-    {-# INLINABLE asin #-}
+    {-# INLINE asin #-}
 
     atan = fmap atan
-    {-# INLINABLE atan #-}
+    {-# INLINE atan #-}
 
     acos = fmap acos
-    {-# INLINABLE acos #-}
+    {-# INLINE acos #-}
 
     sinh = fmap sinh
-    {-# INLINABLE sinh #-}
+    {-# INLINE sinh #-}
 
     tanh = fmap tanh
-    {-# INLINABLE tanh #-}
+    {-# INLINE tanh #-}
 
     cosh = fmap cosh
-    {-# INLINABLE cosh #-}
+    {-# INLINE cosh #-}
 
     asinh = fmap asinh
-    {-# INLINABLE asinh #-}
+    {-# INLINE asinh #-}
 
     atanh = fmap atanh
-    {-# INLINABLE atanh #-}
+    {-# INLINE atanh #-}
 
     acosh = fmap acosh
-    {-# INLINABLE acosh #-}
+    {-# INLINE acosh #-}
 
     (**) = liftA2 (**)
-    {-# INLINABLE (**) #-}
+    {-# INLINE (**) #-}
 
     logBase = liftA2 logBase
-    {-# INLINABLE logBase #-}
+    {-# INLINE logBase #-}
 
 {-| Like 'Fold', but monadic.
 
@@ -306,11 +315,11 @@ instance Monad m => Functor (FoldM m a) where
         done' x = do
             b <- done x
             return $! f b
-    {-# INLINABLE fmap #-}
+    {-# INLINE fmap #-}
 
 instance Monad m => Applicative (FoldM m a) where
     pure b = FoldM (\() _ -> return ()) (return ()) (\() -> return b)
-    {-# INLINABLE pure #-}
+    {-# INLINE pure #-}
 
     (FoldM stepL beginL doneL) <*> (FoldM stepR beginR doneR) =
         let step (Pair xL xR) a = do
@@ -326,7 +335,7 @@ instance Monad m => Applicative (FoldM m a) where
                 x <- doneR xR
                 return $! f x
         in  FoldM step begin done
-    {-# INLINABLE (<*>) #-}
+    {-# INLINE (<*>) #-}
 
 instance Monad m => Profunctor (FoldM m) where
     rmap = fmap
@@ -334,97 +343,97 @@ instance Monad m => Profunctor (FoldM m) where
 
 instance (Monoid b, Monad m) => Monoid (FoldM m a b) where
     mempty = pure mempty
-    {-# INLINABLE mempty #-}
+    {-# INLINE mempty #-}
 
     mappend = liftA2 mappend
-    {-# INLINABLE mappend #-}
+    {-# INLINE mappend #-}
 
 instance (Monad m, Num b) => Num (FoldM m a b) where
     fromInteger = pure . fromInteger
-    {-# INLINABLE fromInteger #-}
+    {-# INLINE fromInteger #-}
 
     negate = fmap negate
-    {-# INLINABLE negate #-}
+    {-# INLINE negate #-}
 
     abs = fmap abs
-    {-# INLINABLE abs #-}
+    {-# INLINE abs #-}
 
     signum = fmap signum
-    {-# INLINABLE signum #-}
+    {-# INLINE signum #-}
 
     (+) = liftA2 (+)
-    {-# INLINABLE (+) #-}
+    {-# INLINE (+) #-}
 
     (*) = liftA2 (*)
-    {-# INLINABLE (*) #-}
+    {-# INLINE (*) #-}
 
     (-) = liftA2 (-)
-    {-# INLINABLE (-) #-}
+    {-# INLINE (-) #-}
 
 instance (Monad m, Fractional b) => Fractional (FoldM m a b) where
     fromRational = pure . fromRational
-    {-# INLINABLE fromRational #-}
+    {-# INLINE fromRational #-}
 
     recip = fmap recip
-    {-# INLINABLE recip #-}
+    {-# INLINE recip #-}
 
     (/) = liftA2 (/)
-    {-# INLINABLE (/) #-}
+    {-# INLINE (/) #-}
 
 instance (Monad m, Floating b) => Floating (FoldM m a b) where
     pi = pure pi
-    {-# INLINABLE pi #-}
+    {-# INLINE pi #-}
 
     exp = fmap exp
-    {-# INLINABLE exp #-}
+    {-# INLINE exp #-}
 
     sqrt = fmap sqrt
-    {-# INLINABLE sqrt #-}
+    {-# INLINE sqrt #-}
 
     log = fmap log
-    {-# INLINABLE log #-}
+    {-# INLINE log #-}
 
     sin = fmap sin
-    {-# INLINABLE sin #-}
+    {-# INLINE sin #-}
 
     tan = fmap tan
-    {-# INLINABLE tan #-}
+    {-# INLINE tan #-}
 
     cos = fmap cos
-    {-# INLINABLE cos #-}
+    {-# INLINE cos #-}
 
     asin = fmap sin
-    {-# INLINABLE asin #-}
+    {-# INLINE asin #-}
 
     atan = fmap atan
-    {-# INLINABLE atan #-}
+    {-# INLINE atan #-}
 
     acos = fmap acos
-    {-# INLINABLE acos #-}
+    {-# INLINE acos #-}
 
     sinh = fmap sinh
-    {-# INLINABLE sinh #-}
+    {-# INLINE sinh #-}
 
     tanh = fmap tanh
-    {-# INLINABLE tanh #-}
+    {-# INLINE tanh #-}
 
     cosh = fmap cosh
-    {-# INLINABLE cosh #-}
+    {-# INLINE cosh #-}
 
     asinh = fmap asinh
-    {-# INLINABLE asinh #-}
+    {-# INLINE asinh #-}
 
     atanh = fmap atanh
-    {-# INLINABLE atanh #-}
+    {-# INLINE atanh #-}
 
     acosh = fmap acosh
-    {-# INLINABLE acosh #-}
+    {-# INLINE acosh #-}
 
     (**) = liftA2 (**)
-    {-# INLINABLE (**) #-}
+    {-# INLINE (**) #-}
 
     logBase = liftA2 logBase
-    {-# INLINABLE logBase #-}
+    {-# INLINE logBase #-}
 
 -- | Apply a strict left 'Fold' to a 'Foldable' container
 fold :: Foldable f => Fold a b -> f a -> b
@@ -543,11 +552,43 @@ sum :: Num a => Fold a a
 sum = Fold (+) 0 id
 {-# INLINABLE sum #-}
 
--- | Computes the product all elements
+-- | Computes the product of all elements
 product :: Num a => Fold a a
 product = Fold (*) 1 id
 {-# INLINABLE product #-}
 
+-- | Compute a numerically stable arithmetic mean of all elements
+mean :: Fractional a => Fold a a
+mean = Fold step begin done
+  where
+    begin = Pair 0 0
+    step (Pair x n) y = Pair ((x * n + y) / (n + 1)) (n + 1)
+    done (Pair x _) = x
+{-# INLINABLE mean #-}
+
+-- | Compute a numerically stable (population) variance over all elements
+variance :: Fractional a => Fold a a
+variance = Fold step begin done
+  where
+    begin = Pair3 0 0 0
+
+    step (Pair3 n mean_ m2) x = Pair3 n' mean' m2'
+      where
+        n'     = n + 1
+        mean'  = (n * mean_ + x) / (n + 1)
+        delta  = x - mean_
+        m2'    = m2 + delta * delta * n / (n + 1)
+
+    done (Pair3 n _ m2) = m2 / n
+{-# INLINABLE variance #-}
+
+{-| Compute a numerically stable (population) standard deviation over all
+    elements
+-}
+std :: Floating a => Fold a a
+std = sqrt variance
+{-# INLINABLE std #-}
+
 -- | Computes the maximum element
 maximum :: Ord a => Fold a (Maybe a)
 maximum = _Fold1 max
@@ -633,6 +674,20 @@ findIndex predicate = Fold step (Left' 0) hush
         _       -> x
 {-# INLINABLE findIndex #-}
 
+
+{-| @(lookup a)@ returns the element paired with the first matching item, or
+    'Nothing' if none matches
+-}
+lookup :: Eq a => a -> Fold (a,b) (Maybe b)
+lookup a0 = Fold step Nothing' lazy
+  where
+    step x (a,b) = case x of
+      Nothing' -> if a == a0
+        then Just' b
+        else Nothing'
+      _ -> x
+{-# INLINABLE lookup #-}
+
 data Pair3 a b c = Pair3 !a !b !c
 
 -- | Pick a random element, using reservoir sampling
@@ -819,18 +874,18 @@ vector = FoldM step begin done
 > impurely Pipes.Prelude.foldM
 >     :: Monad m => FoldM m a b -> Producer a m () -> m b
 
-    Similarly the @ofoldlUnwrap@ and @ofoldMUnwrap@ functions from the
-    @monotraversable@ package are written to interoperate with this library:
+    Other streaming libraries supporting 'purely' and 'impurely' include @io-streams@ and @streaming@. 
+    So for example we have:
 
-> ofoldlUnwrap
->     :: MonoFoldable mono
->     => (x -> Element mono -> x) -> x -> (x -> b) -> mono -> b
+> purely System.IO.Streams.fold_ 
+>     :: Fold a b -> Streams.InputStream a -> IO b
 >
-> ofoldMUnwrap
->     :: (Monad m, MonoFoldable mono)
->     => (x -> Element mono -> m x) -> m x -> (x -> m b) -> mono -> m b
+> impurely System.IO.Streams.foldM_ 
+>     :: FoldM IO a b -> Streams.InputStream a -> IO b
 
-    You can wrap these to accept 'Fold' or 'FoldM', too:
+    The @monotraversable@ package makes it convenient to apply a 
+    'Fold' or 'FoldM' to pure containers that do not allow 
+    a general 'Foldable' instance, like unboxed vectors:
 
 > purely ofoldlUnwrap
 >     :: MonoFoldable mono
@@ -853,8 +908,7 @@ purely_ f (Fold step begin done) = done (f step begin)
 
 -- | Upgrade a monadic fold to accept the 'FoldM' type
 impurely
-    :: Monad m
-    => (forall x . (x -> a -> m x) -> m x -> (x -> m b) -> r)
+    :: (forall x . (x -> a -> m x) -> m x -> (x -> m b) -> r)
     -> FoldM m a b
     -> r
 impurely f (FoldM step begin done) = f step begin done
@@ -901,8 +955,9 @@ simplify (FoldM step begin done) = Fold step' begin' done'
 {- | Shift a 'FoldM' from one monad to another with a morphism such as 'lift' or 'liftIO';
      the effect is the same as 'Control.Monad.Morph.hoist'.
 -}
-hoists :: Monad m => (forall x . m x -> n x) -> FoldM m a b -> FoldM n a b
+hoists :: (forall x . m x -> n x) -> FoldM m a b -> FoldM n a b
 hoists phi (FoldM step begin done) = FoldM (\a b -> phi (step a b)) (phi begin) (phi . done)
+{-# INLINABLE hoists #-}
 
 {-| Allows to continue feeding a 'FoldM' even after passing it to a function
 that closes it.
@@ -925,6 +980,7 @@ _Fold1 step = Fold step_ Nothing' lazy
     step_ mx a = Just' (case mx of
         Nothing' -> a
         Just' x -> step x a)
+{-# INLINABLE _Fold1 #-}
 
 {-| @(premap f folder)@ returns a new 'Fold' where f is applied at each step
 
@@ -1010,6 +1066,26 @@ handles k (Fold step begin done) = Fold step' begin done
     step' = flip (appEndo . getDual . getConst . k (Const . Dual . Endo . flip step))
 {-# INLINABLE handles #-}
 
+{- | @(foldOver f folder xs)@ folds all values from a Lens, Traversal, Prism or Fold with the given folder
+
+>>> foldOver (_Just . both) L.sum (Just (2, 3))
+5
+
+>>> foldOver (_Just . both) L.sum Nothing
+0
+
+> L.foldOver f folder xs == L.fold folder (xs^..f)
+
+> L.foldOver (folded.f) folder == L.fold (handles f folder)
+
+> L.foldOver folded == L.fold
+
+-}
+foldOver :: Handler s a -> Fold a b -> s -> b
+foldOver l (Fold step begin done) =
+  done . flip appEndo begin . getDual . getConst . l (Const . Dual . Endo . flip step)
+{-# INLINABLE foldOver #-}
+
 {-|
 > instance Monad m => Monoid (EndoM m a) where
 >     mempty = EndoM return
@@ -1019,7 +1095,10 @@ newtype EndoM m a = EndoM { appEndoM :: a -> m a }
 
 instance Monad m => Monoid (EndoM m a) where
     mempty = EndoM return
+    {-# INLINE mempty #-}
+
     mappend (EndoM f) (EndoM g) = EndoM (f <=< g)
+    {-# INLINE mappend #-}
 
 {-| A Handler for the upstream input of `FoldM`
 
@@ -1046,12 +1125,26 @@ type HandlerM m a b =
 >
 > handlesM t (f <*> x) = handlesM t f <*> handlesM t x
 -}
-handlesM :: Monad m => HandlerM m a b -> FoldM m b r -> FoldM m a r
+handlesM :: HandlerM m a b -> FoldM m b r -> FoldM m a r
 handlesM k (FoldM step begin done) = FoldM step' begin done
   where
     step' = flip (appEndoM . getDual . getConst . k (Const . Dual . EndoM . flip step))
 {-# INLINABLE handlesM #-}
 
+{- | @(foldOverM f folder xs)@ folds all values from a Lens, Traversal, Prism or Fold monadically with the given folder
+
+> L.foldOverM (folded.f) folder == L.foldM (handlesM f folder)
+
+> L.foldOverM folded == L.foldM
+
+-}
+foldOverM :: Monad m => HandlerM m s a -> FoldM m a b -> s -> m b
+foldOverM l (FoldM step begin done) s = do
+  b <- begin
+  r <- (flip appEndoM b . getDual . getConst . l (Const . Dual . EndoM . flip step)) s
+  done r
+{-# INLINABLE foldOverM #-}
+
 {-|
 > folded :: Foldable t => Fold (t a) a
 >
@@ -1061,6 +1154,7 @@ folded
     :: (Contravariant f, Applicative f, Foldable t)
     => (a -> f a) -> (t a -> f (t a))
 folded k ts = contramap (\_ -> ()) (F.traverse_ k ts)
+{-# INLINABLE folded #-}
 
 {-|
 >>> fold (handles (filtered even) sum) [1..10]
